@@ -1,5 +1,5 @@
 package Data::Babel;
-our $VERSION='1.03';
+our $VERSION='1.10';
 $VERSION=eval $VERSION;         # I think this is the accepted idiom..
 #################################################################################
 #
@@ -21,11 +21,14 @@ use Class::AutoClass;
 use Carp;
 use Graph::Undirected;
 use List::MoreUtils qw(uniq);
+use List::Util qw(min);
 use Hash::AutoHash::Args;
+use Hash::AutoHash::MultiValued;
 use Data::Babel::Config;
 use Data::Babel::IdType;
 use Data::Babel::Master;
 use Data::Babel::MapTable;
+use Data::Babel::HAH_MultiValued;
 
 use vars qw(@AUTO_ATTRIBUTES @OTHER_ATTRIBUTES %SYNONYMS %DEFAULTS %AUTODB);
 use base qw(Data::Babel::Base);
@@ -93,7 +96,7 @@ sub _make_objects {
   @_=@{$_[0]} if 'ARRAY' eq ref $_[0]; # flatten ARRAY
   my @objects;
   for (@_) {
-    push(@objects,$_),next if UNIVERSAL::isa($_,$class);
+    push(@objects,$_),next if ref($_) && $_->isa($class);
     # else let Config handle it
     push(@objects,@{new Data::Babel::Config(file=>$_)->objects($class)});
   }
@@ -113,72 +116,189 @@ sub translate {
   my $self=shift;
   my $args=new Hash::AutoHash::Args(@_);
   # NG 11-01-21: do arg checking here
-  my $missing_args=join(', ',grep {!$args->$_} qw(input_idtype output_idtypes));
-  confess "Required argument(s) $missing_args missing" if $missing_args;
+  # NG 12-10-31: output_idtypes not really required. in any case, it can be empty
+  # my $missing_args=join(', ',grep {!$args->$_} qw(input_idtype output_idtypes));
+  confess "Required argument input_idtype missing" unless $args->input_idtype;
+  $args->output_idtypes([]) unless defined $args->output_idtypes;
   my $ids_args=grep {$args->$_} qw(input_ids input_ids_all);
-  confess "At least one of input_ids or input_ids_all must be set" if $ids_args==0;
+  # NG 12-08-22: okay to omit input_ids; same as input_ids_all=>1
+  # confess "At least one of input_ids or input_ids_all must be set" if $ids_args==0;
   confess "At most one of input_ids or input_ids_all may be set" if $ids_args>1;
   my $sql=$self->generate_query($args);
   my $dbh=$args->dbh || $self->autodb->dbh;
+  # my $results=$args->count? $dbh->selectrow_array($sql): $dbh->selectall_arrayref($sql);
   my $results=$dbh->selectall_arrayref($sql);
   confess "Database query failed:\n$sql\n".$dbh->errstr if $dbh->err;
+  unless ($args->validate) {
+    $results=@$results? $results->[0][0]: 0 if $args->count;
+  } else { 
+    # cases:
+    # 1) no filters. $results contains all valid input_ids. missing ids are invalid
+    # 2) filters. $results contains subset of valid input_ids. have to query db to
+    #    get all valid ones
+    # initial code needed for all cases
+    my $input_ids=$args->input_ids;
+    my @input_ids=ref $input_ids? uniq(@$input_ids): defined $input_ids? ($input_ids): ();
+    my $num_outputs=ref $args->output_idtypes? @{$args->output_idtypes}: 0;
+
+    # %have_id tells which input ids are in result
+    # @missing_ids are input ids not in result - some are valid, some not
+    # %id2valid maps input ids to validity
+    my %have_id=map {$_->[0]=>1} @$results;
+    my @missing_ids;
+    my %id2valid;
+
+    if (!$args->filters) {
+      %id2valid=%have_id;
+    } else {
+      my $input_idtype=$self->_2idtype($args->input_idtype);
+      my $input_colname=_generate_colname($input_idtype);
+      my $input_tablename=$input_idtype->tablename;
+      my $sql=qq(SELECT DISTINCT $input_colname FROM $input_tablename);
+      if (@input_ids) {
+	my @input_ids_sql=map {$dbh->quote($_)} @input_ids;
+	my $where= "WHERE $input_colname IN ".'('.join(', ',@input_ids_sql).')';
+	$sql.="\n$where";
+      }
+      my $valid_ids=$dbh->selectcol_arrayref($sql);
+      confess "Database query failed:\n$sql\n".$dbh->errstr if $dbh->err;
+      %id2valid=map {$_=>1} @$valid_ids;
+      @input_ids=@$valid_ids unless defined $input_ids;
+    }
+    @missing_ids=grep {!$have_id{$_}} @input_ids;
+    push(@$results,map {[$_,$id2valid{$_}||0,(undef)x$num_outputs]} @missing_ids);
+ 
+    # post-processing
+    # 1) count. return number of rows
+    # 2) limit. select subset of rows
+    # 3) count+limit: return min of count, limit
+    if ($args->count) {
+      $results=scalar @$results;
+      $results=min($args->limit,$results) if defined $args->limit;
+    } elsif (defined $args->limit) {
+      @$results=@$results[0..min($args->limit,scalar @$results)-1];
+    }
+  }
   $results;
 }
+sub count {
+  my $self=shift;
+  my $args=new Hash::AutoHash::Args(@_);
+  $args->count(1);
+  $self->translate($args);
+}
+sub validate {
+  my $self=shift;
+  my $args=new Hash::AutoHash::Args(@_);
+  confess "Required argument input_idtype missing" unless $args->input_idtype;
+  $args->validate(1);
+  $args->output_idtypes([$args->input_idtype]) unless defined $args->output_idtypes;
+  $self->translate($args);
+}
 sub generate_query {
   my($self,$args)=@_;
-  my @idtypes=($args->input_idtype,@{$args->output_idtypes});
-  # confess "Not enough types specified" unless @idtypes>1;
+  # be careful about objects vs. names
+  # use variables xxx_idtype for objects, xxx_name for names
+  my $input_idtype=$self->_2idtype($args->input_idtype);
+  my $input_name=_generate_colname($input_idtype);
+  my $input_ids=$args->input_ids;
+  $input_ids=[$input_ids] if defined $input_ids && !ref $input_ids;
+  
+  my $filters=$args->filters;
+  # make sure all filter keys are names and values undef or ARRAYS - simplifies later code
+  $filters=new Data::Babel::HAH_MultiValued $filters;
+  my @filter_keys=keys %$filters;
+  my @filter_idtypes=map {$self->_2idtype($_)} @filter_keys;
+  # my @filter_names=map {$_->name} @filter_idtypes;
+  my @filter_names=map {_generate_colname($_)} @filter_idtypes;
+  $filters=new Data::Babel::HAH_MultiValued
+    (map {$filter_names[$_]=>$filters->{$filter_keys[$_]}} (0..$#filter_keys));
+
+  my @output_idtypes=map {$self->_2idtype($_)} @{$args->output_idtypes};
+  my @output_names=map {$_->name} @output_idtypes;
+  my @idtypes=uniq($input_idtype,@output_idtypes,@filter_idtypes);
   confess "Not enough types specified" unless @idtypes;
-  # idtypes can be specified as objects or names. make sure all idtype names are valid
-  my @bad=grep {!ref $_ && !$self->name2idtype($_)} @idtypes;
-  confess "Invalid idtypes: @bad" if @bad;
-  # convert names to objects
-  # NG 10-08-25: removed 'uniq' since duplicate columns are supposed to be kept
-  # @idtypes=uniq map {ref $_? $_: $self->name2idtype($_)} @idtypes;
-  @idtypes=map {ref $_? $_: $self->name2idtype($_)} @idtypes;
-  my @uniq_idtypes=uniq @idtypes;
- 
+
   my $dbh=$self->autodb->dbh;
-  my $columns_sql=join(', ',map {$_->name} @idtypes);
-  my $input_idtype=$idtypes[0];
+  my @columns=($input_name,@output_names);
+  # splice in VALID column if validating
+  splice(@columns,1,0,'1 AS VALID') if $args->validate;
+  # MySQL doesn't allow duplicate columns in inner queries. sigh..
+  @columns=uniq(@columns) if $args->count;
+  my $columns_sql=join(', ',@columns);
+  
   my $input_master=$input_idtype->master;
-  # start with master if 'informative': explicit || degree>1. always use if single idtype
-  my @join_tables=($input_master->explicit || $input_idtype->degree>1 || @uniq_idtypes==1)?
-    $input_master->tablename: ();
+  # start with input master if 'informative': explicit || degree>1. 
+  # always use if single idtype or user wants all input_ids
+  my @join_tables=
+    ($input_master->explicit || $input_idtype->degree>1 || @idtypes==1 || !defined($input_ids))?
+      $input_master->tablename: ();
   # get rest of query by exploring query graph
-  push(@join_tables,map {$self->id2name($_)} $self->traverse($self->make_query_graph(@idtypes)))
-    if @uniq_idtypes>1;
-  my $join_sql=join(' natural left outer join ',@join_tables);
-  my $input_name=$idtypes[0]->name;
+  push(@join_tables,map {$self->id2name($_)} 
+       $self->traverse($self->make_query_graph(@idtypes))) if @idtypes>1;
+  # add in filter masters for any with histories
+  push(@join_tables,map {$_->tablename} grep {$input_idtype != $_} @filter_idtypes);
+  my $join_sql=join(' NATURAL LEFT OUTER JOIN ',@join_tables);
   # NG 10-08-19: need to quote everything unless we're willing to check the SQL type
   #              'cuz, if column is string and input_id is number, MySQL converts the 
   #              string to a number (not vice versa which makes much more sense !!!!).
   #              since most strings convert to 0, this means that an input_id of 0
   #              matches almost everything
-  # my @input_ids=@{$args->input_ids};
-  # @input_ids=map {$dbh->quote($_)} @input_ids if grep /\D/,@input_ids;
   # NG 11-01-21: add 'translate all'
   my @conds;			# WHERE clauses
-  if ($args->input_ids) {
-    my @input_ids=map {$dbh->quote($_)} @{$args->input_ids};
-    my $in_list_sql='('.join(', ',@input_ids).')'; 
-    push(@conds," $input_name IN $in_list_sql");
+  if (defined $input_ids) {
+    my @input_ids=map {$dbh->quote($_)} @$input_ids;
+    my $cond=@input_ids?
+      " $input_name IN ".'('.join(', ',@input_ids).')': ' FALSE';
+    push(@conds,$cond);
+  }
+  # NG 12-08-24: support filters
+  # NG 12-09-21: handle undefs
+  for my $filter_name (@filter_names) {
+    my $cond;
+    my $filter_ids=$filters->$filter_name;
+    if (!defined $filter_ids) {
+      $cond="$filter_name IS NOT NULL";
+    } elsif (@$filter_ids) {
+      my @cond;
+      my @defined_ids=map {$dbh->quote($_)} grep {defined $_} @$filter_ids;
+      push(@cond," $filter_name IN ".'('.join(', ',@defined_ids).')') if @defined_ids;
+      push(@cond,"$filter_name IS NULL") if @defined_ids!=@$filter_ids;
+      $cond=join(' OR ',@cond);
+      $cond="($cond)" if @cond>1;
+    } else {			# it's filter=>undef[]
+      $cond="FALSE";
+    }
+    push(@conds,$cond);
   }
   # NG 10-11-10: skip rows whose output columns are all NULL
-  if (@idtypes>1)  {
-    my @output_idtypes=@idtypes[1..$#idtypes]; # input idtype is always 0th
-    my $sql_not_null=join(' OR ',map {$_->name.' IS NOT NULL'} @output_idtypes);
+  # NG 12-11-23: but keep these rows if validating
+  if (@output_names && !$args->validate)  {
+    my $sql_not_null=join(' OR ',map {"$_ IS NOT NULL"} @output_names);
     push(@conds,"($sql_not_null)");
   }
   my $sql="SELECT DISTINCT $columns_sql FROM $join_sql";
   $sql.=' WHERE '.join(' AND ',@conds) if @conds;
+
   # NG 10-11-08: support limit. based on DM's change
+  # NG 12-11-23: if validating, have to do limit in code...
   my $limit=$args->limit;
   confess "Invalid limit: $limit" if defined $limit && $limit=~/\D/;
-  $sql.=" LIMIT $limit" if defined $limit;
+  $sql.=" LIMIT $limit" if defined $limit && !$args->validate;
+
+  # NG 12-09-23: support count
+  # Note: 'AS T' needed at end of query because mySQL sees inner select as defining derived
+  #       table and requires every derived table have alias
+  # NG 12-11-23: if validating, have to do count in code...
+  $sql=qq(SELECT COUNT(*) FROM ($sql) AS T) if $args->count && !$args->validate;
+  
   $sql;
 }
-
+# input is IdType object. handles history
+sub _generate_colname {
+  my $idtype=shift;
+  !$idtype->history? $idtype->name: '_X_'.$idtype->name;
+}
 # we're using a modified (bipartite) schema graph. nodes are IdTypes and MapTables.
 # edges go between MapTables and the IdTypes they contain
 # use persistent ids for nodes (rather than objects) so it will work when fetched from db 
@@ -263,26 +383,31 @@ sub make_imps {
     $idtype->master($master);	     # connect new Master to its IdType
   }
 }
-# sub make_imps {
-#   my $self=shift;
-#   my $schema_graph=$self->schema_graph;
-#   my @joiners=grep {$schema_graph->degree($_->id)>1} @{$self->idtypes};
-#   my %idtype2master=map {$_->idtype->name => $_} @{$self->masters};
-#   my @need_imps=grep {!$idtype2master{$_->name}} @joiners;
-#   for my $idtype (@need_imps) {
-#     my $idtype_name=$idtype->name;
-#     my @maptables=map {$self->id2object($_)} $schema_graph->neighbors($idtype->id);
-#     # my @maptable_names=map {$_->name} @maptables;
-#     my $inputs=join("\n",map {$_->namespace.'/'.$_->name} @maptables);
-#     my $query=join("\nUNION\n",map {"SELECT $idtype_name FROM ".$_->name} @maptables);
-#     my $master=new Data::Babel::Master
-#       (name=>$idtype->name.'_master',inputs=>$inputs,query=>$query,implicit=>1,
-#        babel=>$self,idtype=>$idtype);
-#     push(@{$self->masters},$master); # connect new Master to Babel
-#     $self->name2master($master);     # add new Master to name hash
-#     $idtype->master($master);	     # connect new Master to its IdType
-#   }
-# }
+# NG 12-09-27: added load_implicit_masters (just stub for now)
+#              in stub, create tables, since otherwise tests crash.
+sub load_implicit_masters {
+  my $self=shift;
+  my $dbh=$self->dbh;
+  my @implicits=grep {$_->implicit} @{$self->masters};
+  for my $master (@implicits) {
+    my $tablename=$master->tablename;
+    my $idtype=$master->idtype;
+    my $column_name=$idtype->name;
+    my $column_sql_type=$idtype->sql_type;
+    my $query=$master->query;
+    my $sql=$master->view?
+      qq(CREATE VIEW $tablename): 
+	qq(CREATE TABLE $tablename ($column_name $column_sql_type));
+    $sql.=" AS\n$query";
+    $dbh->do(qq(DROP VIEW IF EXISTS $tablename));
+    $dbh->do(qq(DROP TABLE IF EXISTS $tablename));
+    $dbh->do($sql);
+    if (my $errstr=$dbh->errstr) {
+      confess "Unable to create Master $tablename: $errstr";
+    }
+    $dbh->do(qq(ALTER TABLE $tablename ADD INDEX ($column_name)));
+  }
+}
 
 sub show { 
   my $self=shift;
@@ -306,7 +431,7 @@ sub show {
 }
 # can be called as function or method
 sub show_graph {
-  my $graph=UNIVERSAL::isa($_[0],'Graph')? $_[0]: $_[1];
+  my $graph=ref($_[0]) && $_[0]->isa('Graph')? $_[0]: $_[1];
   print '  ',join("\n  ",map {_edge_str($graph,$_)} _sort_edges($graph->edges)),"\n";
 }
 sub _sort_edges {
@@ -318,7 +443,6 @@ sub _edge_str {
   my($v0,$v1)=@$edge;
   $v0 le $v1? "$v0 - $v1": "$v1 - $v0";
 }
-
 # checks (1) schema graph is tree; (2) all IdTypes covered
 sub check_schema {
   my $self=shift;
@@ -346,7 +470,7 @@ sub check_contents {confess "check_contents NOT YET IMPLEMENTED"};
 # can be called as function or method. $root is a maptable or master
 my %seen;
 sub traverse {
-  my($root,$graph)=UNIVERSAL::isa($_[1],'Graph')? @_[0,1]: @_[1,2];
+  my($root,$graph)=ref($_[0]) && $_[0]->isa('Graph')? @_[0,1]: @_[1,2];
   %seen=($root=>$root);
   _traverse($root,$graph);
 }
@@ -379,6 +503,49 @@ sub _name2object {
   }
 }
 
+# _2idtype used in generate_query
+sub _2idtype {
+  my $self=shift;
+  if (ref $_[0]) {
+    confess "Invalid idtype $_[0]" unless $_[0]->isa('Data::Babel::IdType');
+    return $_[0];
+  }
+  # else may be name or stringified ref
+  unless ($_[0]=~/^Data::Babel::IdType=HASH\(0x\w+\)$/) {
+    my $idtype=$self->name2idtype($_[0]);
+    confess "Invalid idtype $_[0]" unless ref($idtype) && $idtype->isa('Data::Babel::IdType');
+    return $idtype;
+  }
+  # code to convert stringified ref adapted from http://stackoverflow.com/questions/1671281/how-can-i-convert-the-stringified-version-of-array-reference-to-actual-array-ref?rq=1
+  # CAUTION: will segfault if bad string passed in!
+  require B;
+  my($hexaddr)=$_[0]=~/.*(0x\w+)/;
+  my $idtype=bless(\(0+hex $hexaddr), "B::AV")->object_2svref;
+  confess "Invalid filter idtype $_[0]" unless $idtype=~/^Data::Babel::IdType=HASH\(0x\w+\)$/;
+  $idtype;
+}
+# _2idtype_name NOT USED
+# sub _2idtype_name {
+#   my $self=shift;
+#   if (ref $_[0]) {
+#     confess "Invalid idtype $_[0]" unless UNIVERSAL::isa($_[0],'Data::Babel::IdType');
+#     return $_[0]->name;
+#   }
+#   # else may be name or stringified ref
+#   unless ($_[0]=~/^Data::Babel::IdType=HASH\(0x\w+\)$/) {
+#     my $idtype=$self->name2idtype($_[0]);
+#     confess "Invalid idtype $_[0]" unless UNIVERSAL::isa($idtype,'Data::Babel::IdType');
+#     return $_[0];
+#   }
+#   # code to convert stringified ref adapted from http://stackoverflow.com/questions/1671281/how-can-i-convert-the-stringified-version-of-array-reference-to-actual-array-ref?rq=1
+#   # CAUTION: will segfault if bad string passed in!
+#   require B;
+#   my($hexaddr)=$_[0]=~/.*(0x\w+)/;
+#   my $idtype=bless(\(0+hex $hexaddr), "B::AV")->object_2svref;
+#   confess "Invalid filter idtype $_[0]" unless $idtype=~/^Data::Babel::IdType=HASH\(0x\w+\)$/;
+#   $idtype->name;
+# }
+
 # TODO: belongs in some Util
 sub flatten {map {'ARRAY' eq ref $_? @$_: $_} @_;}
 
@@ -395,7 +562,7 @@ Data::Babel - Translator for biological identifiers
 
 =head1 VERSION
 
-Version 1.03
+Version 1.10
 
 =head1 SYNOPSIS
 
@@ -420,26 +587,43 @@ Version 1.03
   # open database containing real data
   my $dbh=DBI->connect("dbi:mysql:database=test",undef,undef);
 
-  # translate several Entrez Gene ids to other types
   # CAUTION: rest of SYNOPSIS assumes you've loaded the real database somehow
+  # translate several Entrez Gene ids to other types
   my $table=$babel->translate
-    (dbh=>$dbh,
-     input_idtype=>'gene_entrez',
+    (input_idtype=>'gene_entrez',
      input_ids=>[1,2,3],
-     output_idtypes=>[qw(gene_symbol gene_ensembl
-                         transcript_refseq transcript_ensembl
-                         chip_affy probe_affy chip_lumi probe_lumi)]);
+     output_idtypes=>[qw(gene_symbol gene_ensembl chip_affy probe_affy)]);
   # print a few columns from each row of result
   for my $row (@$table) {
     print "Entrez gene=$row->[0]\tsymbol=$row->[1]\tEnsembl gene=$row->[2]\n";
   }
+  # same translation but limit results to Affy hgu133a
+  my $table=$babel->translate
+    (input_idtype=>'gene_entrez',
+     input_ids=>[1,2,3],
+     filters=>{chip_affy=>'hgu133a'},
+     output_idtypes=>[qw(gene_symbol gene_ensembl chip_affy probe_affy)]);
   # generate a table mapping all Entrez Gene ids to UniProt ids
   my $table=$babel->translate
     (input_idtype=>'gene_entrez',
-     input_ids_all=>1,
      output_idtypes=>[qw(protein_uniprot)]);
   # convert to HASH for easy programmatic lookups
   my %gene2uniprot=map {$_[0]=>$_[1]} @$table;
+  
+  # count number of Entrez Gene ids represented on Affy hgu133a
+  my $count=$babel->count
+    (input_idtype=>'gene_entrez',filters=>{chip_affy=>'hgu133a'});
+
+  # tell which input ids are valid
+  my $table=$babel->validate
+    (input_idtype=>'gene_entrez',
+     input_ids=>[1,2,3]);
+  # print validity status of each
+  for my $row (@$table) {
+    my($input_id,$valid,$current_id)=@$row;
+    print "Entrez gene $input_id is ",
+          ($valid? "valid with current value $current_id": 'invalid'),"\n";
+  }
 
 =head1 DESCRIPTION
 
@@ -453,10 +637,11 @@ the shared types.
 The principal method is 'translate' which converts identifiers of one
 type into identifiers of one or more output types.  In typical usage,
 you call 'translate' with a list of input ids to convert.  You can
-also call it without any input ids (and with the special option
+also call it without any input ids (or with the special option
 'input_ids_all' set) to generate a complete mapping of the input type
 to the output types.  This is convenient if you want to hang onto the
-mapping for repeated use.
+mapping for repeated use.  You can also filter the output based on
+values of other identifier types.
 
 CAVEAT: Some features of Data::Babel are overly specific to the
 procedure we use to construct the underlying Babel database.  We note
@@ -474,7 +659,7 @@ The main components of a Data::Babel object are
 
 =item * a master list of valid values for the type, and 
 
-=item * optionally, a history mapping old values to current ones [NOT YET IMPLEMENTED]
+=item * optionally, a history mapping old values to current ones
 
 =back
 
@@ -492,6 +677,16 @@ of MapTables must, of course, cover all the IdTypes.  The ensemble of
 MapTables must also be non-redundant as explained in L<Technical
 details>.
 
+MapTables must always contain current identifiers, even for IdTypes
+that have histories (more precisely, for IdTypes whose Masters have
+histories). The query or program that loads the database is
+responsible for mapping old identifiers to current ones (presumably
+via the history).
+
+'translate' checks the input IdType to see if its Master has history
+information. If so, 'translate' automatically applies the history to
+all input ids.  It does the same for filters.
+
 You need not explicitly define Masters for all IdTypes; Babel
 will create 'implicit' Masters for any IdTypes lacking explicit
 ones. An implicit Master has a list of valid identifiers but no
@@ -562,9 +757,32 @@ Master
   QUERY
 
 The section name is the Master name; the name of the IdType is the
-same but without the '_master'. The parameters are used by our
-database construction procedure and may not be useful in other
-settings.
+same but without the '_master'. The 'inputs' and 'query' parameters
+are used by our database construction procedure and may not be useful
+in other settings.
+
+The next example illustrates a Master that includes history information.
+
+  [gene_entrez_master]
+  inputs=<<INPUTS
+  MainData/GeneInformation MainData/GeneHistory
+  INPUTS
+  query=<<QUERY
+  SELECT old_locus_link_eid AS _ANY_gene_entrez, locus_link_eid AS gene_entrez
+  FROM gene_information LEFT OUTER JOIN gene_history 
+    ON locus_link_eid=new_locus_link_eid
+  QUERY
+  history=1
+
+A Master without history is implemented as a one column table whose
+column has the same name as the IdType.
+
+A Master with history is implemented as a two column table: one column
+has the same name as the IdType and the other has the prefix '_X_'
+prepended to the IdType. The '_X_' column contains ids that were valid
+in the past or are valid now. Each row maps the '_X_' id to its
+current value, if any, or NULL. For ids that are valid now, the table
+contains a row in which the '_X_' and current versions are the same.
 
 MapTable
 
@@ -605,35 +823,34 @@ construction procedure and may not be useful in other settings.
 
 =head2 Input ids that do not connect to any outputs
 
-The 'translate' method does not return any output for input
-identifiers that do not connect to any identifiers of the desired
-output types.  In other words, 'translate' never returns output rows
-in which the output columns are all NULL.
+By default, the 'translate' method does not return any output for
+input identifiers that do not connect to any identifiers of the
+desired output types; these are output rows in which the output
+columns are all NULL. You can instruct 'translate' to include these
+rows in the result by setting the 'validate' option.
 
-An input identifier can fail to connect for several reasons: 
+An input identifier can fail to connect for two reasons: 
 
 =over 2
 
-=item 1. The identifier does not exist in the Master table for the
-input IdType; this generally means that the input id is not valid.
+=item 1. The identifier is not valid, in other words, it does not
+exist in the Master table for the input IdType.
 
-=item 2. The identifier exists in the Master table for the input
-IdType (hence is valid) but is not present in any MapTables; this is
-rare, because it means the identifer is valid but does not participate
-in any relationships.
-
-=item 3. The identifier exists in the Master table for the input
-IdType and one or more MapTables, but the rows that match the input
-contain NULLs for all output IdTypes; this is normal and simply means
-that the input doesn't connect to any ids of the desired output types.
+=item 2. The identifier is valid but doesn't doesn't connect to any
+ids of the desired output types. This is normal.
 
 =back
 
+If you set the 'validate' option, the output will contain at least one
+row for each input identifier, and an additional column that indicates
+whether each input identifier is valid.
+
 If no output IdTypes are specified, 'translate' returns a row
 containing one element, namely, the input identifier, for each input
-id that exists in the corresponding Master table.  This is the only
-way at present for the application to distinguish non-existent ids
-from ones that exist but don't connect.
+id that exists in the corresponding Master table. If the 'validate'
+option is set, the output will contain one row for each input
+identifier; this is essentially a (possibly re-ordered) copy of the
+input list with duplicates removed.
 
 =head2 Technical details
 
@@ -744,36 +961,228 @@ The available class attributes are
  Usage   : $table=$babel->translate
                      (input_idtype=>'gene_entrez',
                       input_ids=>[1,2,3],
+                      filters=>{chip_affy=>'hgu133a'},
                       output_idtypes=>[qw(transcript_refseq transcript_ensembl)],
                       limit=>100)
-           -- OR --
-           $table=$babel->translate
-                     (input_idtype=>'gene_entrez',
-                      input_ids_all=>1,
-                      output_idtypes=>[qw(transcript_refseq transcript_ensembl)],
-                      limit=>100000)
- Function: Translate the input ids into ids of the output types
+ Function: Translate the input ids to ids of the output types
  Returns : table represented as an ARRAY of ARRAYS. Each inner ARRAY is one row
-           of the result; the first element of each is an input id, the rest are
-           outputs in the same order as output_idtypes
+           of the result. The first element of each row is an input id. If the
+           validate option is set, the second element of each row indicates
+           whether the input id is valid. The rest are outputs in the same order
+           as output_idtypes
  Args    : input_idtype   name of Data::Babel::IdType object or object
-           input_ids      ARRAY of ids to be translated.  It is an error to set
-                          both input_ids and input_ids_all.
-           input_ids_all  If true, all ids of the input type are translated. We
-                          recommend that 1 be used as the true value. See Notes 
-                          below. It is an error to set both input_ids and
-                          input_ids_all. 
+           input_ids      id or ARRAY of ids to be translated. If absent or
+                          undef, all ids of the input type are translated. If an
+                          empty array, ie, [], no ids are translated and the 
+                          result will be empty.
+           input_ids_all  boolean. If true, all ids of the input type are
+                          translated. Same as omitting input_ids or setting it
+                          to undef but more explicit.
+           filters        HASH or ARRAY of conditions limiting the output; see 
+                          below.
            output_idtypes ARRAY of names of Data::Babel::IdType objects or
                           objects
-           limit          maximum number of rows to retrieve (optional)
- Notes   : Duplicate columns are retained. 
-           Does not return output rows in which the output columns are all NULL.
-           If no output idtypes are specified, returns rows for which the input
-           id exists in the corresponding Master table.
-           The order of output rows is arbitrary.
-           At present, input_ids_all may be set to any true value but in future,
-           we may interpret the value differently. 
-           It is an error to set both input_ids and input_ids_all. 
+           validate       boolean. If true, the output will contain at least one
+                          row for each input id and an additional column 
+                          indicating whether the input id is valid.
+           limit          maximum number of rows to retrieve
+           count          boolean. If true, return number of output rows rather 
+                          than the rows themselves. Equivalent to 'count'
+                          method.
+
+=head3 Notes on translate
+
+=over 2
+
+=item * 'translate' retains duplicate output columns.
+
+=item * The order of output rows is arbitrary.
+
+=item * If input_ids is absent or undef, it translates all ids of the
+input type.
+
+=item * Duplicate input_ids are ignored.
+
+=item * If input_ids is an empty ARRAY, ie, [], the result will be
+empty.
+
+=item * It is an error to set both input_ids and input_ids_all.
+
+=item * It is legal but odd to specify a filter on the input
+idtype. This effectively computes the intersection of the input and
+filter ids.
+
+=item * Input and filter ids can be old (valid in the past) or current
+(valid now). Output ids are always current.
+
+=item * By default, 'translate' does not return rows in which the
+output columns are all NULL. Setting 'validate' changes this and
+ensures that every input id will appear in the output.
+
+=item * If 'count' and 'limit' both set, the result is the number of
+output rows after the limit is applied and will always be <= the
+limit.
+
+=item * If 'validate' and 'limit' both set, the result may not contain
+all input ids if to do so would produce more rows than the limit. This
+defeats one of the purposes of 'validate', namely to ensure that all
+input ids appear in the output.
+
+=item * If 'count' and 'validate' both set, the result is the number
+of output rows including ones added by 'validate', ie, rows with in
+which all output columns are NULL.
+
+=item * If 'validate' and 'filters' both set, the result may contain
+input ids excluded by the filter. These rows will have NULLs in all
+output columns.
+
+=item * If no output idtypes are specified, the output will contain
+one row for each valid input id (by default) or one row for each id
+whether valid or not (if 'validate' is set). 
+
+=back
+
+=head3 Filters
+
+The 'filters' argument is a HASH or ARRAY of types and values. The
+types can be names of Data::Babel::IdType objects or objects
+themselves. The values can be single ids, ARRAYs of ids, or undef; the
+ARRAYs may also contain undef. For example
+
+  filters=>{chip_affy=>'hgu133a'}
+  filters=>{chip_affy=>['hgu133a','hgu133plus2']}
+  filters=>{chip_affy=>['hgu133a','hgu133plus2'],pathway_kegg_id=>4610}
+  filters=>{chip_affy=>['hgu133a','hgu133plus2'],pathway_kegg_id=>undef}
+  filters=>{chip_affy=>'hgu133a',pathway_kegg_id=>[undef,4610]}
+
+If the argument is an ARRAY, it is possible for the same type to
+appear multiple times in which case the values are combined.  For example,
+
+  filters=>[chip_affy=>'hgu133a',chip_affy=>'hgu133plus2']
+
+is equivalent to 
+
+  filters=>{chip_affy=>['hgu133a','hgu133plus2']}
+
+If a filter value is an empty ARRAY, ie, [], the result will be empty.
+
+If a filter value is undef, all ids of the given type are acceptable.
+This limits the output to rows for which the filter type is not
+NULL. For example,
+
+  $babel->translate(input_idtype=>'gene_entrez',
+                    filters=>{pathway_kegg_id=>undef},
+                    output_idtypes=>[qw(gene_symbol)])
+
+generates a table of all Entrez Gene ids and gene symbols which appear in
+any KEGG pathway.
+
+Including undef in an ARRAY lets the output contain rows for which the
+filter type is NULL.  For example,
+
+  $babel->translate(input_idtype=>'gene_entrez',
+                    filters=>{pathway_kegg_id=>[undef,4610]},
+                    output_idtypes=>[qw(gene_symbol)])
+
+generates a table of all Entrez Gene ids and gene symbols which either
+appear in KEGG pathway 4610 or appear in no KEGG pathway.
+
+CAUTION: undef has opposite semantics depending on whether it's the
+only value for a filter type or whether it's one of several.
+
+=head3 Histories
+
+'translate' automatically applies histories, when they exist, to input
+and filter ids. In other words, input and filter ids can be ones that
+were valid in the past but are not valid now. Output ids, however, are
+always current.
+
+CAUTION: If the input type is also used as an output, the result can
+contain rows in which the output id does not equal the input id. This
+will occur if the input id is old and is mapped to a different current
+value.  Likewise, if a filter type is used as an output, the result
+can contain rows in which the output id does not match the filter.
+
+=head2 count
+
+ Title   : count 
+ Usage   : $number=$babel->count
+                     (input_idtype=>'gene_entrez',
+                      input_ids=>[1,2,3],
+                      filters=>{chip_affy=>'hgu133a'},
+                      output_idtypes=>[qw(transcript_refseq transcript_ensembl)])
+ Function: Count number of output rows that would be generated by 'translate'
+ Returns : number
+ Args    : same as 'translate'
+
+'count' is a wrapper for L<translate> that sets the 'count' argument to a true value.
+
+=head2 validate
+
+ Title   : validate 
+ Usage   : $table=$babel->validate
+                     (input_idtype=>'gene_entrez',
+                      input_ids=>[1,2,3])
+ Function: Tell which input ids are valid now or in the past, and the mapping 
+           from old to current values
+ Returns : table represented as an ARRAY of ARRAYS. Each inner ARRAY is one row
+           of the result. If output_idtypes is omiited (the usual case), the 
+           elements of each row are
+             0) input id as given
+             1) validity status. 1 for valid; 0 for invalid
+             2) current value of the id or undef if it has no current value; may
+                be the same as the original id
+           If output_idtypes is set, the result is ther same as 'translate' with
+           the 'validate' option set
+ Args    : input_idtype   name of Data::Babel::IdType object or object
+           input_ids      id or ARRAY of ids to be translated. If absent or
+                          undef, all ids of the input type are translated. If an
+                          empty array, ie, [], no ids are translated and the 
+                          result will be empty.
+           input_ids_all  boolean. If true, all ids of the input type are
+                          translated. Same as omitting input_ids or setting it
+                          to undef but more explicit.
+           output_idtypes optional and usually omitted. ARRAY of names of 
+                          Data::Babel::IdType objects or objects. If set,
+                          equivalent to calling 'translate' with the 'validate'
+                          option set
+           limit          maximum number of rows to retrieve
+
+'validate' looks up the given input ids in the Master tables for the
+given input type and returns a table indicating which ids are
+valid. For types with history information, the method also
+indicates the current value of the id. For types that have no history,
+the current value will always equal the given id if the id is valid.
+
+'validate' can also retrieve a complete table of valid ids (along with
+history information) for the type.
+
+'validate' is a wrapper for L<translate> that sets the 'validate'
+argument to a true value and the output_idtypes argument to the
+input_idtype.  All other 'translate' arguments (filters, count) are
+legal here and work but are of dubious value.
+
+=head3 Notes on validate
+
+=over 2
+
+=item * For rows whose validity status is 1 (valid), the given id and
+current value indicate the history: if the elements are equal, the
+given id is current; else if the current value is defined, the given
+id has been replaced by the new one; else the given id was valid in
+the past but has no current value.
+
+=item * For types that have no history, all valid ids are current. If
+the given id is valid, the given id and current value will be equal;
+else the current value will be undef.
+
+=item * For rows whose status is 0 (invalid), the current value will
+always be undef.
+
+=item * The 'translate' arguments 'filters' and 'count' are legal here
+and work but are of dubious value.
+
+=back
 
 =head2 show
 
@@ -804,6 +1213,23 @@ The available class attributes are
  Returns : boolean
  Args    : none
 
+=head2 load_implicit_masters
+
+ Title   : load_implicit_masters
+ Usage   : $babel->load_implicit_masters
+ Function: Creates database structures for implicit Masters. 
+ Returns : nothing useful
+ Args    : none
+
+Babel creates 'implicit' Masters for any IdTypes lacking explicit
+ones. An implicit Master has a list of valid identifiers and could be
+implemented as a view over all MapTables containing the IdType. In the
+current implementation, we use views for IdTypes contained in single
+MapTables but construct actual tables for IdTypes contained in
+multiple MapTables. 
+
+This method must be called after the real database is loaded.
+
 =head2 Finding component objects by name or id & related
 
 Objects have names and ids: names are strings like 'gene_entrez' and
@@ -900,7 +1326,14 @@ The available object attributes are
   id            name prefixed with 'idtype', eg, 'idtype:::gene_entrez'
   master        Data::Babel::Master object for this IdType
   maptables     ARRAY of Data::Babel::MapTable objects containing this IdType
-  display_name  human readable name, eg, 'Entrez Gene ID'
+  external      boolean indicating whether this is a regular external ID or one
+                intended for internal use
+  internal      opposite of external
+  history       boolean indicating whether this IdType's Master contains history
+                information
+  tablename     name of this IdType's Master's table
+  display_name  human readable name, eg, 'Entrez Gene ID'; for internal 
+                identifiers, a warning is appended to the end
   referent      the type of things to which this type of identifier refers
   defdb         the database, if any, which assigns identifiers
   meta          meta-type: eid (meaning synthetic), symbol, name, description
@@ -920,7 +1353,6 @@ The available class attributes are
  Returns : number
  Args    : none
 
-
 =head1 METHODS AND ATTRIBUTES OF COMPONENT CLASS Data::Babel::Master
 
 =head2 new
@@ -955,11 +1387,13 @@ The available class attributes are
 The available object attributes are
 
   name          eg, 'gene_entrez_master' 
-  id            name prefixed with 'master', eg, 'master:::gene_entrez_master'
+  id            name prefixed with 'master::', eg, 'master:::gene_entrez_master'
   idtype        Data::Babel::IdType object for which this is the Master
   implicit      boolean indicating whether Master is implicit
   explicit      opposite of implicit
   view          boolean indicating whether Master is implemented as a view
+  history       boolean indicating whether Master contains history information.
+  tablename     synonym for name
   inputs, namespace, query
                 used by our database construction procedure
 
